<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\libs\Util\SMTP;
use Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\QuerySelector;

class Mym00107FaleConoscoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $crudService;
    protected $smtpService;
    protected $connection;
    protected $fileServerFilesUrl;
    
	public function __construct($smtpService, $crudService, $connection, $fileServerFilesUrl){
	    $this->SMTPService = $smtpService;
	    $this->crudService = $crudService;
	    $this->connection  = $connection;
	    $this->fileServerFilesUrl = $fileServerFilesUrl;
	    $this->templatesDir = __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'Templates' . DIRECTORY_SEPARATOR;
	}
	
    public function checkEmailRemetente(DTO\Request $request, DTO\Response $response){   
        try{
              $row = (array) $request->getRow();
                $conditions = array(
                    array(
                        "columnName" => "CDFILIAL",
                        "operator" => "=",
                        "value" => $row['CDFILIAL']
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $this->crudService->getEnvironment()->getNrOrg()
                    )
                );
            $rows = $this->crudService->getFilteredDataSet('check_email_filial', $conditions)->getRows();
            $email = count($rows) > 0 ? $rows[0]['DSEMAILFILI'] : null;
            $response->addDataSet(new DataSet("emailOperador", array($email)));
        }  catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    //send email
    public function getFileContentUtf8($fileName) {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }
    
    public function buildEmailLogoTemplate($message, $urlLogo, $idAplicativo, $question) {
        $content = $this->getFileContentUtf8($this->templatesDir . "mail_logo.html");
        
        $title = "Tecfood ".$idAplicativo;

        $search = array(
            "%title%",
            "%question%",
            "%message%",
            "%logoURL%",
            "%idAplicativo%"
        );
        $replace = array(
            $title,
            $question,
            $message,
            $urlLogo['URLLOGO'],
            $idAplicativo
        );
        return str_replace($search, $replace, $content);
    }
    
    private function getFileUri($row) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
        
        $queryOracle = <<<SQL
            SELECT URLLOGO FROM APPCONFIG
            WHERE
                IDAPLICATIVO = :IDAPLICATIVO
                AND NOT EXISTS (
                    SELECT
                        URLLOGO
                    FROM
                        QRCODELOCALPESQ
                    WHERE
                        CDFILIAL = :CDFILIAL
                        AND NVL(NRLOCALPESQ, 0) = NVL(:NRLOCALPESQ, 0)
                )
            UNION ALL
            SELECT
                MAX(URLLOGO) AS URLLOGO
            FROM
                QRCODELOCALPESQ
            WHERE
                CDFILIAL = :CDFILIAL
                AND NVL(NRLOCALPESQ, 0) = NVL(:NRLOCALPESQ, 0)
SQL;
        $queryPostgre = <<<SQL
            SELECT URLLOGO AS "URLLOGO" FROM APPCONFIG
            WHERE
                IDAPLICATIVO = :IDAPLICATIVO
                AND NOT EXISTS (
                    SELECT
                        URLLOGO
                    FROM
                        QRCODELOCALPESQ
                    WHERE
                        CDFILIAL = :CDFILIAL
                        AND COALESCE(NRLOCALPESQ, 0) = COALESCE(:NRLOCALPESQ, 0)
                )
            UNION ALL
            SELECT
                MAX(URLLOGO) AS URLLOGO
            FROM
                QRCODELOCALPESQ
            WHERE
                CDFILIAL = :CDFILIAL
                AND COALESCE(NRLOCALPESQ, 0) = COALESCE(:NRLOCALPESQ, 0)
SQL;
        $params = array (
            "IDAPLICATIVO" => strtoupper($row['IDAPLICATIVO']),
            "NRLOCALPESQ" => $row['NRLOCALPESQ'],
            "CDFILIAL" => $row['CDFILIAL']
        );
        
        $arrayQueries = array (
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
        
        try{
            // $std = $this->connection->prepare($query);
            // $std->execute($params);
            // $rowResult = $std->fetch();
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        return $connection->fetchAll($query, $params);
        
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    
    private function getFilial($row){
  
        $query = "
            
               SELECT 
                NVL(DSLOGINSMTP, DSEMAILFILI) AS username,
                DSSMTPEMAILF AS host,
                CDSENHEMAILF AS password,
                CASE
                    WHEN IDUTTLSEMAIL = 'S' 
                    THEN 'tls' 
                    WHEN IDUTSSLEMAIL = 'S' 
                    THEN 'ssl' 
                    ELSE '' 
                END AS smtpSecure, 
                NVL(DSEMAILFILI, DSLOGINSMTP) AS email_From,
                CASE 
                    WHEN NVL(DSLOGINSMTP, DSEMAILFILI) IS NOT NULL AND 
                    CDSENHEMAILF IS NOT NULL 
                    THEN 'S' 
                    ELSE 'N' 
                    END AS auth
                FROM ENDEFILI 
                WHERE CDFILIAL = :CDFILIAL 
                AND IDTPENDEFILI = 'P' AND
                DSSMTPEMAILF IS NOT NULL
        ";
        
        
        $params = [
            "CDFILIAL" => $row['CDFILIAL']
        ];
        
        try{
            $std = $this->connection->prepare($query);
            $std->execute($params);
            $rowResult = $std->fetch();
            
            if(!empty($rowResult)){
                $rowResult['username'] = $rowResult['USERNAME'];
                $rowResult['host'] = $rowResult['HOST'];
                $rowResult['password'] = $rowResult['PASSWORD'];
                $rowResult['smtpSecure'] = $rowResult['SMTPSECURE'];
                $rowResult['from'] = $rowResult['EMAIL_FROM'];
                $rowResult['smtpAuth'] = $rowResult['AUTH'];
                    
                unset($rowResult['USERNAME']);
                unset($rowResult['HOST']);
                unset($rowResult['PASSWORD']);
                unset($rowResult['SMTPSECURE']);
                unset($rowResult['EMAIL_FROM']);
                unset($rowResult['AUTH']);
                
                if($rowResult['smtpAuth'] == 'S')
                    $rowResult['smtpAuth'] = true;
                else
                    $rowResult['smtpAuth'] = false;
            }
    
            return $rowResult;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function sendEmail(DTO\Request $request, DTO\Response $response){
        try{
            $row = (array) $request->getRow();
            $question = $row['QUESTION'];
            $message = $row['MESSAGE'];
            $address = $row['EMAILCLIENTE'];
            $idAplicativo = $row['IDAPLICATIVO'];
            $logo = $this->getFileUri($row);
            $logo['URLLOGO'] = $this->fileServerFilesUrl.$logo[0]['URLLOGO'];
            
            if($idAplicativo != "MyQuest"){

                if($idAplicativo == "MyMenu"){
                    
                    $smtpParametros = $this->getFilial($row);
                    if(!empty($smtpParametros)){
                        $paramsAux = Utilities::getSmtpSettingsFromAddress($smtpParametros['host']);
                        $smtpParametros['host'] = $paramsAux['host'];
                        $smtpParametros['port'] = $paramsAux['port'] ?? null;
                        $smtpParametros['password'] = ($smtpParametros['password']) ? Utilities::getDelphiPasswordEncrypted($smtpParametros['password'], false) : '';
                        
                        $SMTPServiceAux = new SMTP($smtpParametros);
                        $this->SMTPService = $SMTPServiceAux;
                    }
                }
                $mailBody = $this->buildEmailLogoTemplate($message, $logo, $idAplicativo, $question);
                $this->SMTPService->setSubject('Resposta Fale Conosco MyMenu');
                $this->SMTPService->setFromName('SAC MyMenu');
                $this->SMTPService->sendMail($mailBody, $address);
           }
            
        }catch(\Exception $e){
            $message = $e->getMessage();
            $errorMessage = "";
            
            if(stripos($message, "SMTP connect") !== false) { 
                $errorMessage = "SMTP_CONNECT_FAILED"; 
            } else {
                $errorMessage = "EMAIL_SENT_ERROR"; 
            }
            
            $response->setError(new Error($errorMessage,500));
            $response->addDataSet(new DataSet("sendEMailError", array('ERROR' => $e->getMessage())));
        }
    }
}
